#!/bin/sh

#
# Version = VERSION in /etc/vdr/plugins/mylinvdr.conf
#
# Erstellt eine Sicherungskopie basierend auf den Original-Daten von LinVDR 0.7 und
# der auf dem PC installierten Updates, nderungen, etc. Das fertige Archiv wird dann
# in ein DVD-ISO umgewandelt und anschlieend auf DVD+R(W) gebrannt. Diese DVD kann
# anschlieend wie die Original LinVDR 0.7 Boot-ISO zur Installation benutzt werden und
# richtet das gesamte System wieder so ein wie zuvor. Nur Daten, welche in der Datei
# /etc/vdr/plugins/mylinvdr.conf mit exclude angegeben sind, werden nicht gesichert, um z.B.
# die eigenen Videos und MP3s, etc auszunehmen.
#
# Skript erstellt Dank Hilfe von vielen aus dem www.vdr-portal.de Forum, u.a. von
# Sascha Theissen (44892), Ecki (viper67), Markus Kuechler (Mahlzeit), decembersoul & mannih
#
# Bei Weiterentwicklungen wird um Zusendung einer Kopie gebeten an linvdr@theissen-legal.com
#
# Viel Spa mit dem Skript!
#


#=============================================== define functions ================================#
function message
{
	case "$OUTPUT" in
  		vdr)
  			$SVDRPSEND mesg "$1"
  			sleep $SLEEPTIME
  			;;
  		shell)
  			echo "$1"
  			;;
  		logger)
  			logger "$1"
  			;;
	esac
	echo "$1" >> $LOGFILE
}

function debug
{
	if [ "$1" == "$DEBUG" ]
	then
		echo "	$2"
		echo "	----------------------------------------------"
	fi
}

function usage
{
	echo ""
    echo " This is mylinvdr.sh, version $VERSION"
	echo "MyLinVDR.sh [-m {iso|cd|dvd}] [-o {vdr|shell|logger}] [-b {fast|full|no}] [-d {yes|no}] [-v {1|2|full}]"
	echo ""
	echo "-m:	Mode"
	echo "	iso:	create only ISO image and save it to %ISOPATH/$ISONAME"
	echo "	cd:	    create ISO image and burn it to a CD"
	echo "	dvd:	create ISO image and burn it to a DVD"
	echo ""
	echo "-o:	Show output at"
	echo "	vdr:	VDR OSD"
	echo "	shell:	STDOUT at local shell"
	echo "	logger:	Syslog"
	echo ""
	echo "-b:	Blank RW Media"
	echo "	fast:	Fast erase of RW Media"
	echo "	full:	Full erase of RW Media"
	echo "	no:	    Do not erase RW Media"
	echo ""
	echo "-d:	Delete image file ($ISOFILE and linvdr-0.7.tar.gz)"
	echo "	yes:	Delete all temporary images (not used for mode iso)"
	echo "	no:	    Do not delete all temporary images (overrides default settings for mode)"
	echo ""
	echo "-v:	Verbose output"
	echo "	1:  	Loglevel 1 (not yet used)"
	echo "	2:	    Loglevel 2"
	echo "	full:	Loglevel full (set -x)"
	echo ""
	echo "-h:	show this page"
	echo "-help	show this page"
	echo ""
	echo ""
}

function clearlog
{
	echo "" > $LOGFILE
	echo "" > $ERRLOGFILE
}

#=============================================== Parameters ==========================================#
while getopts ':m:v:o:b:d:h:' opt
do
        case $opt in
                m)			#Mode: iso | cd | dvd
                        case "$OPTARG" in
                        	iso)		#Only ISO-Image without burning
                        					MODE=iso
                        					BURN=0
                        					DELETE=${DELETE:=0}
                        					;;
                        	cd)			#Burn MyLinVDR to CD-R(W)
                        					MODE=cd
                        					BURN=1
                        					DELETE=${DELETE:=1}
                        					;;
                        			  		        *)			usage
																	exit 1
																	;;
                        	dvd)		#Burn MyLinVDR to DVD+/-R(W)
                        					MODE=dvd
                        					BURN=1
                        					DELETE=${DELETE:=1}
                        					;;
													*)			usage
																	exit 1
																	;;
                        esac
                        ;;
                o)			#Output: vdr | shell
                        case "$OPTARG" in
                        	vdr)		#Output Messages to VDR
                        					OUTPUT=vdr
                        					;;
                        	shell)	#Output Messages to shell (STDOUT)
                        					OUTPUT=shell
                        					;;
                        	logger)	#Output Messages to Syslog
                        					OUTPUT=logger
                        					;;
													*)			usage
																	exit 1
																	;;
                        esac
                        ;;
	              b)			#Blank: fast | full
                        case "$OPTARG" in
                        	fast)		#Perform a fast RW erase
                        					BLANK=fast
                        					;;
                        	full)		#Perform a full RW erase
                        					BLANK=full
                        					;;
                        	no)			#Do not perform any RW erase (Default)
                        					BLANK=no
                        					;;
													*)			usage
																	exit 1
																	;;
                        esac
                        ;;
	              d)			#Delete ISO Image/ linvdr-0.7.tar.gz to save discspace
												case "$OPTARG" in
													yes)		#Delete temp files
																	DELETE=1
																	;;
													no)			#Do not delete temp files, overrides other settings
																	DELETE=0
																	;;
													*)			usage
																	exit 1
																	;;
												esac
												;;
	              v)			#Verbose/Debug level
                        case "$OPTARG" in
                        	1)			#Debug Level 1
                        					DEBUG=1
                        					;;
                        	2)			#Debug Level 2
                        					DEBUG=2
                        					;;
                        	full)		#Debug Level full (set -x)
                        					DEBUG=0
                        					set -x
                        					;;
                        	*)			usage
                        					exit 1
                        					;;
												esac
												echo "Debugmode: $DEBUG"
												;;
	              a)			#Append log to existing logfile or clear logfile
												case "$OPTARG" in
													yes)		#Append to existing log
																	if [ -n $LOGFILE ]
																	then
																		echo "" > $LOGFILE
																	fi
																	if [ -n $ERRLOGFILE ]
																	then
																		echo "" > $ERLOGFILE
																	fi
																	;;
													no)			#Clear logfiles
																	clearlog
																	;;
													*)			usage
																	exit 1
																	;;
												esac
												;;
								h)			#Display help
												OUTPUT=shell
												usage
												exit 0
												;;
								*)			OUTPUT=shell
												usage
												exit 0
												;;
        esac
done


debug 2 "functions defined"

#=============================================== main ================================================#

# load configuration file
if [ -e /etc/vdr/plugins/mylinvdr.conf ]
then
	. /etc/vdr/plugins/mylinvdr.conf
else
	message "No config file found. Aborting"
	exit 1
fi

debug 2 "config loaded"

# output version info
message "running mylinvdr.sh version $VERSION"

# Default values for Variables
declare -x MODE=${MODE:=dvd}
declare -x OUTPUT=${OUTPUT:=vdr}
declare -x BURN=${BURN:=1}
declare -x DELETE=${DELETE:=1}
declare -x BLANK=${BLANK:=no}
declare -x DEBUG=${DEBUG:=0}
declare -x SLEEPTIME=${SLEEPTIME:=3}
declare -x LOGFILE=${LOGFILE:=/var/log/mylinvdr.log}
declare -x ERRLOGFILE=${ERRLOGFILE:=/var/log/mylinvdr.err}
declare -x LANGUAGE=${LANGUAGE:=de_DE}

TAR=${TAR:=/bin/tar}
MKISOFS=${MKISOFS:=/usr/bin/mkisofs}
CDRECORD=${CDRECORD:=/usr/bin/cdrecord}
GROWISOFS=${GROWISOFS:=/usr/bin/growisofs}
DVDRWFORMAT=${DVDRWFORMAT:=/usr/bin/dvd+rw-format}
SVDRPSEND=${SVDRPSEND:=/usr/bin/svdrpsend.pl}

debug 2 "Variables set to default if not defined yet"
debug 2 "OUTPUT: $OUTPUT"
debug 2 "BURN: $BURN"
debug 2 "DELETE: $DELETE"
debug 2 "BLANK: $BLANK"
debug 2 "MODE: $MODE"

# load language file
if [ -e /etc/vdr/plugins/mylinvdr/$LANGUAGE.lang ]
then
	. /etc/vdr/plugins/mylinvdr/$LANGUAGE.lang
else
	message "No language file found. Aborting"
	exit 2
fi

debug 2 "language file loaded"

debug 2 "start backup"
message "$LANG_OUTPUT_ISO_CREATE"

# Prueft, ob $BOOTISO schon existiert, falls nicht, werden die Daten von der Original- LinVDR-CD
# angefordert, das Verzeichnis erstellt und die Daten kopiert, danach das Skript weiter abgearbeitet
if [ ! -e $BOOTISO/LinVDR ]
then
	debug 2 "copy LinVDR files from CD"
	
	# Dateien kopieren erforderlich
	mkdir -p $BOOTISO
	message "$LANG_OUTPUT_NEED_ORIG_CD"
	read
	mount $MOUNTPATH
	# Ist es die richtige CD?
	while [ ! -d $MOUNTPATH/LinVDR ]
	do
		debug 2 "orig CD not provided"
		eject $BURNDEVICE
		message "$LANG_OUTPUT_NOT_ORIG_CD"
		read
		mount $MOUNTPATH
	done
	debug 2 "copy files..."
	cp -ar $MOUNTPATH/isolinux $BOOTISO
	cp -ar $MOUNTPATH/LinVDR $BOOTISO
	cp /etc/vdr/plugins/mylinvdr.message.txt $BOOTISO/isolinux
	eject $BURNDEVICE
	message "$LANG_OUTPUT_CHANGE_DISC"
	read
fi

if [ -e $BOOTISO/LinVDR/linvdr-0.7.tar.gz ]
then
	debug 2 "delete linvdr-0.7.tar.gz"
	rm $BOOTISO/LinVDR/linvdr-0.7.tar.gz
fi

#Boese hacks um die Verzeichnisstruktur von /pub und /tmp zu erhalten:
debug 2 "touch pubs"
touch /pub/export/.mylinvdr
touch /pub/images/.mylinvdr
touch /pub/mp3/.mylinvdr
touch /pub/mplayer/.mylinvdr
touch /pub/web/.mylinvdr
touch /tmp/.mylinvdr
touch /video0/.mylinvdr
touch /video1/.mylinvdr
touch /vtx/.mylinvdr

# Baue die Excludeliste aus den beiden Konfigurationsparametern zusammen
for dir in $EXCLUDEDIRS
do
	EXCLUDE="$EXCLUDE --exclude=$dir"
done
debug 2 "excludelist builded: $EXCLUDE"
for file in $EXCLUDEFILES
do
	EXCLUDE="$EXCLUDE --exclude=$file"
done
debug 2 "fileexcludelist builded: $EXCLUDE"

# Erstellen des gezippten Tar-Archivs mit aktuellen Daten.
# Ausgenommen Dateien und Verzeichnisse aus EXCLUDEDIRS und EXCLUDEFILES
message "$LANG_OUTPUT_CREATE_TAR"
TARCMD="$TAR czvplf $BOOTISO/LinVDR/linvdr-0.7.tar.gz $EXCLUDE $INCLUDEDIRS $INCLUDEFILES"
debug 2 "tar: $TARCMD"
eval $TARCMD >>$LOGFILE 2>>$ERRLOGFILE
TAROK=$?
debug 2 "tar was running: $TAROK"

if [ $TAROK -eq 0 ]
then
	if [ -e $ISOPATH/$ISOFILE ]
	then
		debug 2 "ISO present, move to .bak"
		message "$LANG_OUTPUT_ISO_PRESENT"
		mv $ISOPATH/$ISOFILE $ISOPATH/$ISOFILE.bak
	fi
	debug 2 "run mkisofs"
	message "$LANG_OUTPUT_CREATE_ISO"
	MKISOFSCMD="$MKISOFS -o $ISOPATH/$ISOFILE -b isolinux/isolinux.bin -no-emul-boot -boot-load-size 4 -boot-info-table -joliet-long -RJ -r $BOOTISO/"
	eval $MKISOFSCMD
	MKISOFSOK=$?
else
	debug 2 "error while tar was running"
	message "$LANG_ERROR_CREATE_TAR"
	echo "Aborted after TAR NOT OK \($TAROK\) at `date`" >> $LOGFILE
	exit 3
fi

if [ $MKISOFSOK -eq 0 ]
then
	debug 2 "mkisofs successful"
	case "$MODE" in
		iso)	debug 2 "iso mode- end now"
					message "$LANG_OUTPUT_ISO_CREATED"
					echo "Stop mylinvdr at `date`" >> $LOGFILE
					exit 0
					;;
		cd)		debug 2 "cd mode $BLANK"
					case "$BLANK" in
						fast)	BLANKCMD="$CDRECORD --dev=$CD_BURN_DEVICE --blank=fast gracetime=1"
                									;;
						full)	BLANKCMD="$CDRECORD --dev=$CD_BURN_DEVICE --blank=all gracetime=1"
                									;;
						no)		message "$LANG_OUTPUT_NO_BLANK"
									;;
					esac
					BURNCMD="$CDRECORD --dev=$CD_BURN_DEVICE $CD_BURN_BURNFREE --speed=$CD_BURN_SPEED $CD_BURN_EJECT -tao gracetime=1 $ISOPATH/$ISOFILE"
                    debug 2 "BLANKCMD: $BLANKCMD"
					debug 2 "BURNCMD: $BURNCMD"
					;;
		dvd)	debug 2 "dvd mode $BLANK"
					case "$BLANK" in
						fast)	BLANKCMD="$DVDRWFORMAT -force -blank $BURNDEVICE"
									;;
						full)	BLANKCMD="$DVDRWFORMAT -force -blank=full $BURNDEVICE"
									;;
						no)		message "$LANG_OUTPUT_NO_BLANK"
									BLANKCMD="/bintrue"
									;;
					esac
					BURNCMD="$GROWISOFS -dvd-compat -use-the-force-luke=tty -Z $BURNDEVICE=$ISOPATH/$ISOFILE"
					debug 2 "BLANKCMD: $BLANKCMD"
					debug 2 "BURNCMD: $BURNCMD"
					;;
	esac
	if [ "x$BLANK" == "xno" ]
	then
		BLANKCMDOK=0
	else
		message "$LANG_OUTPUT_BLANK_RW"
		eval $BLANKCMD
		BLANKCMDOK=$?
	fi
	debug 2 "erasing RW: $BLANKCMDOK"
	
	if [ $BLANKCMDOK -eq 0 ]
	then
		debug 2 "Burn ISO"
		message "$LANG_OUTPUT_BURN_ISO"
		eval $BURNCMD
		BURNCMDOK=$?
		debug 2 "BURNCMDOK: $BURNCMDOK"
		if [ $BURNCMDOK -eq 0 ]
		then
			if [ $DELETE -eq 1 ]
			then
				debug 2 "delete temp files"
				rm -f $ISOPATH/$ISOFILE
				rm -f $BOOTISO/LinVDR/linvdr-0.7.tar.gz
				message "$LANG_OUTPUT_BURN_COMPLETE_IMAGE_DELETED"
				eject $BURNDEVICE
				echo "Stop mylinvdr at `date`" >> $LOGFILE
				exit 0
			else
				debug 2 "do not delete temp files"
				message "$LANG_OUTPUT_BURN_COMPLETE"
				eject $BURNDEVICE
				echo "Stop mylinvdr at `date`" >> $LOGFILE
				exit 0
			fi
		else
			message "$LANG_ERROR_BURN"
			echo "Aborted after BURN $MODE NOT OK at `date`" >> $LOGFILE
  			exit 6
		fi
	else
		message "$LANG_ERROR_BLANK"
		echo "Aborted after BLANK $MODE NOT OK at `date`" >> $LOGFILE
		exit 5
	fi
else
	message "$LANG_ERROR_CREATE_ISO"
	echo "Aborted after MKISOFS NOT OK \($MKISOFSOK\) at `date`" >> $LOGFILE
	exit 4
fi
